import argparse
from pathlib import Path
import os

def main():
    parser = argparse.ArgumentParser("imface cli for image vector")
    parser.add_argument("-v", "--version", help="version", default=False, action="store_true")
    
    subparsers = parser.add_subparsers(dest="command")
    #generate vector represent
    represent_parser = subparsers.add_parser("represent")
    represent_parser.add_argument("-p", "--path", help="path image file", required=True)
    represent_parser.add_argument("-d", "--detector", help="detector", required=True)
    represent_parser.add_argument("-t", "--threshold", help="Face Detector Threshold", required=False, default=0.75)
    represent_parser.add_argument('-check_blurry', action='store_true')

    #generate vector selfie
    selfie_parser = subparsers.add_parser("selfie")
    selfie_parser.add_argument("-p", "--path", help="path image file", required=True)
    selfie_parser.add_argument("-d", "--detector", help="detector", required=True)
    #generate image album
    generate_parser = subparsers.add_parser("generate-crop-img")
    generate_parser.add_argument("-p", "--path", help="path to image file", required=True)
    generate_parser.add_argument("-o", "--output", help="directory to save image", required=True)
    generate_parser.add_argument("-d", "--detector", help="face detector", required=True)
    generate_parser.add_argument("-t", "--threshold", help="Face Detector Threshold", required=False, default=0.75)
    generate_parser.add_argument('-check_blurry', action='store_true')

    args = parser.parse_args()

    version =  "0.0.0.4.2"
    if args.version:
        os.environ.setdefault("DEEPFACE_HOME", "/app")
        print(version + str(os.getenv("DEEPFACE_HOME", default=str(Path.home()))))
        exit(0)

    elif args.command == "represent":
        try:
            os.environ.setdefault("DEEPFACE_HOME", "/app")

            from imfacesnap.utils import deepface_util as utils
            embed = utils.get_embedding_vector(path=args.path, detector=args.detector, threshold=float(args.threshold), check_blurry=args.check_blurry)
            print(embed)
        except Exception as e:
            print("error " + repr(e))
            raise SystemExit(1)

    elif args.command == "selfie":
        try:
            os.environ.setdefault("DEEPFACE_HOME", "/app")
            
            from imfacesnap.utils import deepface_util as utils
            data = utils.extract_face(path=args.path, detector=args.detector)
            if len(data) > 1:
                print("error only allowed one face")
                raise SystemExit(1)
            else:
                print(data[0]['embedding'])
        except Exception as e:
            print("error" + repr(e))
            raise SystemExit(1)

    elif args.command == "generate-crop-img":
        try:
            os.environ.setdefault("DEEPFACE_HOME", "/app")
            if not os.path.exists(args.output):
                os.makedirs(args.output)

            from imfacesnap.utils import deepface_util as utils
            file_names = utils.generate_faces_image(path=args.path, album_dir=args.output, detector=args.detector, threshold=float(args.threshold), check_blurry=args.check_blurry)
            print(file_names)
        except Exception as e:
            print("error " + repr(e))
            raise SystemExit(1)

if __name__ == "__main__":
    main()


import os
from deepfacey import DeepFace
import uuid
import cv2
from imface.utils.tools import get_blurry_score

models = [
  "VGG-Face", 
  "Facenet", 
  "Facenet512", 
  "OpenFace", 
  "DeepFace", 
  "DeepID", 
  "ArcFace", 
  "Dlib", 
  "SFace",
]
BLURRYNESS_THRESHOLD = 100

def get_embedding_vector(path: str, detector: str, threshold: float=0.75, check_blurry: bool=False):
    image_np = cv2.imread(path)
    embed = []
    extracted_faces = DeepFace.represent(
        image_np,
        model_name=models[2],
        enforce_detection=True,
        detector_backend=detector,
    )
    for face in extracted_faces:
        face_area = face['facial_area']
        x, y, w, h = int(face_area['x']), int(face_area['y']), int(face_area['w']), int(face_area['h'])
        crop_face = image_np[y:y+h,x:x+w]
        temp = {
            'embedding': face['embedding'],
            'blurry_score': get_blurry_score(crop_face),
            'face_confidence': face['face_confidence']
        }
        embed.append(temp)
    return embed

def extract_face(path: str, detector: str):
    data = DeepFace.represent(
        path,
        model_name=models[2],
        enforce_detection=True,
        detector_backend=detector,
    )
    return data

def generate_faces_image(path: str, album_dir: str, detector: str, threshold: float=0.75, check_blurry: bool=False):
    image_names = []
    image_np = cv2.imread(path)

    extracted_faces = DeepFace.extract_faces(
        img_path=image_np,
        enforce_detection=True,
        detector_backend=detector,
        align=True,
    )
    for face in extracted_faces:
        face_area = face['facial_area']
        x, y, w, h = int(face_area['x']), int(face_area['y']), int(face_area['w']), int(face_area['h'])
        crop_face = image_np[y:y+h,x:x+w]
        if face['confidence'] >= threshold:
            name = uuid.uuid4()
            target_name = os.path.join(album_dir, f"{name}.jpg")
            if check_blurry:
                if get_blurry_score(crop_face) < BLURRYNESS_THRESHOLD:
                    continue
            cv2.imwrite(target_name, crop_face)
            image_names.append(target_name)
    return image_names



from setuptools import setup, find_packages
from setuptools.command.install import install
from pathlib import Path
import gdown
import os


def get_deepface_home():
    """Get the home directory for storing weights and models.

    Returns:
        str: the home directory.
    """
    os.environ.setdefault("DEEPFACE_HOME", "/app") # This LINE cause error when we try to install in local environment, because APP folder is not initialized
    return str(os.getenv("DEEPFACE_HOME", default=str(Path.home())))

def initialize_folder():
    """Initialize the folder for storing weights and models.

    Raises:
        OSError: if the folder cannot be created.
    """
    home = get_deepface_home()
    deepface_homepath = home + "/.deepface"
    weights_path = deepface_homepath + "/weights"

    if not os.path.exists(deepface_homepath):
        os.makedirs(deepface_homepath, exist_ok=True)
        print("Directory ", home, "/.deepface created")

    if not os.path.exists(weights_path):
        os.makedirs(weights_path, exist_ok=True)
        print("Directory ", home, "/.deepface/weights created")



home_dir = get_deepface_home()
def download_weights():
    #url of the weights that need to be downloaded first
    facenet512_url = "https://github.com/serengil/deepface_models/releases/download/v1.0/facenet512_weights.h5"
    yolov8_url = "https://drive.google.com/uc?id=1UX8saXoKt36H9uMsZIPJt4jk3NoDvNTL"
    #path of the weights will be saved
    facenet512_weight_path = str(home_dir) + "/.deepface/weights/facenet512_weights.h5"
    yolov8_weight_path = str(home_dir) +"/.deepface/weights/yolov8m-face.onnx"
    
    if os.path.isfile(facenet512_weight_path) != True:
        print("facenet512_weights.h5 will be downloaded...")
        gdown.download(facenet512_url, facenet512_weight_path, quiet=False)

    if os.path.isfile(yolov8_weight_path) != True:
        print("yolov8 onnx will be downloaded...")
        gdown.download(yolov8_url, yolov8_weight_path, quiet=False)
        
class CustomInstallCommand(install):
    def run(self):
        install.run(self)
        initialize_folder()
        download_weights()

with open("README.md", "r") as file:
    description = file.read()

requirements = ["deepfacesnap==0.0.2"]

setup(
    name='imfacesnap',
    version='0.0.0.3',
    install_requires=requirements,
    packages=find_packages(),
    include_package_data=True,
    entry_points={"console_scripts": ["imfacesnap=imfacesnap.main:main"]},
    author="Kevin Snap",
    license="License :: OSI Approved :: MIT License",
    classifiers=[
        "Programming Language :: Python",
    ],
    long_description=description,
    long_description_content_type="text/markdown",
    python_requires=">=3.5.5",
    cmdclass={
        'install': CustomInstallCommand,
    },
)